unit untDXFdemo;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, ExtCtrls, math, fpvectorial, dxfvectorialreader;

type

  { TfrmDXFdemo }

  TfrmDXFdemo = class(TForm)
    Image1: TImage;
    OpenDialog1: TOpenDialog;
    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure Image1DblClick(Sender: TObject);

    {Graphics from disk, Graphics in/out stream}
    function PlaatjeFilToMem(aFile: TFilename; var aFileNaa,aFileExt: String; var aStream: TMemoryStream; var aMsg: String): Boolean;
    function PlaatjeCalcExte(var aFileExt: String; aStream: TMemoryStream; var aMsg: String): Boolean;
    function PlaatjeMemInImage(aImage: TImage; aExtent: String; aMem: TMemoryStream; var aMsg: String): Boolean;
    function LoadFromVECStream(aStream: TStream; var aImage: TImage; aType: String; var aMsg: String): Boolean;
    function LoadStreamInVecDoc(var aVecDoc: TvVectorialDocument; aStream: TStream; aType: String; var aMsg: String): Boolean;
    function CalcVecPageSizes(var aVecDoc: TvVectorialDocument; var aVecPag: TvPage; aImage: TImage; var aPagFac: Double; var aWidMar,aTopMar: Integer; var aMsg: String): Boolean;
    function PainVecPageCanva(aVecPag: TvPage; aCanvas: TCanvas; aCanFac: Double; aMarLef,aMarTop: Integer; var aMsg: String): Boolean;

  private
    { private declarations }
  public
    { public declarations }
    aStream          : TMemoryStream;
    aFileNaa,aFileExt:        String;
  end;

var
  frmDXFdemo: TfrmDXFdemo;

implementation

{$R *.lfm}

{ TfrmDXFdemo }

procedure TfrmDXFdemo.FormCreate(Sender: TObject);
begin
  aStream   := TMemoryStream.Create;
  aFileNaa  := '';
  aFileExt  := '';
end;

procedure TfrmDXFdemo.Image1DblClick(Sender: TObject);
var lOK     : Boolean;
    aMsg    : String;
begin
  lOK                   := True;
  aMsg                  := '';
  aFileNaa              := '';
  aFileExt              := '';
  if OpenDialog1.Execute   then
  begin
    if lOK     then lOK := PlaatjeFilToMem(OpenDialog1.FileName,       aFileNaa,aFileExt,aStream,aMsg);
    if lOK and ((aFileNaa = '') or (aFileExt = '')) then lOK := PlaatjeCalcExte(aFileExt,aStream,aMsg);
    if lOK     then lOK := PlaatjeMemInImage(                   Image1,         aFileExt,aStream,aMsg);
  end;
  if (not lOK) then        ShowMessage(aMsg);
end;

procedure TfrmDXFdemo.FormResize(Sender: TObject);
var lOK     : Boolean;
    aMsg    : String;
begin
  aMsg                  := '';
  lOK                   := PlaatjeMemInImage(Image1,aFileExt,aStream,aMsg);
  if (not lOK) then        ShowMessage(aMsg);
end;

{gets file from disk and loads it into a stream}
function TfrmDXFdemo.PlaatjeFilToMem(aFile: TFilename; var aFileNaa,aFileExt: String; var aStream: TMemoryStream; var aMsg: String): Boolean;
var lOK: Boolean;
begin
   lOK       := True;
   try
     aStream.LoadFromFile(aFile);
     if lOK then aFileNaa := ExtractFileNameOnly(aFile);
     if lOK then aFileExt := ExtractFileExt(     aFile);
   except
     on E:Exception do
     begin
        lOK  := False;
        aMsg := 'Error loading Stream from File.' + #10 + E.Message;
     end;
   end;
   Result    := lOK;
end;

{checks in stream what type of file is offered}
function TfrmDXFdemo.PlaatjeCalcExte(var aFileExt: String; aStream: TMemoryStream; var aMsg: String): Boolean;
var buf :    word;
    lOK : Boolean;
begin
  lOK            := True;
  buf            := 0;
  if (aFileExt = '') and (aStream.Size > 0) then
  begin
    try
      if aStream.read(buf, sizeof(buf)) <> sizeOf(buf) then exit;
      aStream.Position := 0;
      if buf      = $D8FF then aFileExt := '.JPG' else
      if buf      = $4D42 then aFileExt := '.BMP' else
//    if buf      = 8224  then aFileExt := '.DXF' else
      if buf      = 18759 then aFileExt := '.GIF' else
      if buf      = 18761 then aFileExt := '.TIF' else
      if buf      = 20617 then aFileExt := '.PNG' else
//    if buf      = 20517 then aFileExt := '.PDF' else
      begin
        aFileExt := IntToStr(buf);
        lOK      := False;
        aMsg     := 'Not presentation possible';
      end;
    except
      on E:Exception do
      begin
         aMsg    := 'Error checking Buffers in Stream  ' + #10 + E.Message;
         lOK     := False;
      end; { on }
    end;
  end;
  Result         := lOK;
end;


function TfrmDXFdemo.PlaatjeMemInImage(aImage: TImage; aExtent: String; aMem: TMemoryStream; var aMsg: String): Boolean;
var  lOK: Boolean;
begin
   lOK             := True;
   try
     begin
       aMem.Position   := 0;
       aImage.Picture  := nil;
       case LowerCase(aExtent)     of
       '.dxf' : if (aMem.Size > 0) then lOK := LoadFromVECStream(aMem,aImage,'dxf',aMsg);
       '.pdf' : if (aMem.Size > 0) then lOK := LoadFromVECStream(aMem,aImage,'pdf',aMsg);
       else
         if (aMem.Size  > 0)       then aImage.Picture.LoadFromStream(aMem);
       end;
     end;
   except
     on E:Exception do
     begin
        aMsg    := 'Error loading picture in Canvas ' + #10 + E.Message;
        lOK     := False;
     end; { on }
   end;
   Result          := lOK;
end;

function TfrmDXFdemo.LoadFromVECStream(aStream: TStream; var aImage: TImage; aType: String; var aMsg: String): Boolean;
var  lOK     : Boolean;
     aMarTop : Integer;
     aMarLef : Integer;
     aCanFac : Double;
     aVecDoc : TvVectorialDocument;
     aVecPag : TvPage;
begin
  lOK                := True;
  aVecDoc            := Nil;
  aVecPag            := Nil;
  aMarTop            := 0;
  aMarLef            := 0;
  aCanFac            := 1;
  try
    lOK              := LoadStreamInVecDoc(aVecDoc,aStream,aType,aMsg);
    if lOK then lOK  := CalcVecPageSizes(aVecDoc,aVecPag,aImage,aCanFac,aMarLef,aMarTop,aMsg);
    if lOK and (aType = 'dxf') then
                lOK  := PainVecPageCanva(aVecPag,aImage.Canvas,aCanFac,aMarLef,aImage.Height-aMarTop,aMsg);
  finally
    aVecDoc.Free;
  end;
  Result             := lOK;
end;

function TfrmDXFdemo.LoadStreamInVecDoc(var aVecDoc: TvVectorialDocument; aStream: TStream; aType: String; var aMsg: String): Boolean;
var lOK : Boolean;
begin
  lOK           := True;
  try
    aVecDoc     := TvVectorialDocument.Create;
  except
    on E:Exception do
    begin
       lOK      := False;
       aMsg     := 'Error creating Vectorial document' + #10 + E.Message;
    end;
  end;
  try
    case atype of
    'dxf' : aVecDoc.ReadFromStream(aStream,vfDXF);
    'pdf' : aVecDoc.ReadFromStream(aStream,vfPDF);
    end;
  except
    on E:Exception do
    begin
       lOK      := False;
       aMsg     := 'Error translating file with due filter ' + aType + #10 + E.Message;
    end;
  end;
  Result        := lOK;
end;

function TfrmDXFdemo.CalcVecPageSizes(var aVecDoc: TvVectorialDocument; var aVecPag: TvPage; aImage: TImage; var aPagFac: Double; var aWidMar,aTopMar: Integer; var aMsg: String): Boolean;
var lOK     : Boolean;
    aSouFac : Double;
    aTarFac : Double;
begin
  lOK                  := True;
  aWidMar              := 0;
  aTopMar              := 0;
  try
    begin
      aVecPag           := aVecDoc.GetPage(0);
      aVecPag.CalculateDocumentSize;
      {the smaller the factor, the narrower, hence the bigger the factor, the wider}
      aSouFac           := aVecPag.MaxX/aVecPag.MaxY;     {factor of source graphic}
      aTarFac           := aImage.Width/aImage.Height;    {factor of target graphic}
      {if the targetfactor is smaller, the width is the problem, else the height is}
      if aTarFac        <= aSouFac then
        if aVecPag.MaxX <> 0 then aPagFac:= aImage.Width /aVecPag.MaxX else aPagFac:= 1
      else
        if aVecPag.MaxY <> 0 then aPagFac:= aImage.Height/aVecPag.MaxY else aPagFac:= 1;
      {calculate a white-margin to the top or the left}
      aTopMar           := ceil((aImage.Height - round(aVecPag.MaxY * aPagFac))/2);
      aWidMar           := ceil((aImage.Width  - round(aVecPag.MaxX * aPagFac))/2);
    end;
  except
    on E:Exception do
    begin
       lOK      := False;
       aMsg     := 'Error calculating sizes and margins ' + #10 + E.Message;
    end;
  end;
  if lOK and Assigned(aVecPag) then aVecPag.BackgroundColor := TColorToFPColor(clWhite);
  if lOK and Assigned(aVecPag) then aImage.Canvas.FillRect(0,0,aImage.Width,aImage.Height);
  Result := lOK;;
end;

function TfrmDXFdemo.PainVecPageCanva(aVecPag: TvPage; aCanvas: TCanvas; aCanFac: Double; aMarLef,aMarTop: Integer; var aMsg: String): Boolean;
var lOK     : Boolean;
    i       : Integer;
    aEntity : TvEntity;
begin
  lOK       := True;
  try
    for i   := 0 to aVecPag.GetEntitiesCount-1 do
    begin
      aEntity                            := aVecPag.GetEntity(i);
      aVecPag.RenderInfo.BackgroundColor := aVecPag.BackgroundColor;
      aEntity.Render(aCanvas,aVecPag.RenderInfo,aMarLef,aMarTop,aCanFac,-aCanFac);
    end;
  except
    on E:Exception do
    begin
       lOK  := False;
       aMsg := 'Error presenting entities from a DXF file' + #10 + E.Message;
    end;
  end;
  Result    := lOK;
end;



end.

